#! /usr/bin/perl -w
#
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2002-2003 SuSE Linux AG Nuernberg, Germany.  
#                         All rights reserved.
#
#
# $Id: ldapupdate40.pl,v 1.1 2003/05/06 11:37:52 varkoly Exp $
#
BEGIN {
    push @INC,"/srv/www/lib";
}

use strict;
use POSIX;
use Net::LDAP qw(:all);
use Net::LDAP::LDIF;
use File::Temp qw(tempfile);
use Getopt::Long;
use Utils;

$Getopt::Long::error = 0;
my $baseDN = "";
my $CYRUSPW = "";
my $ldaphost = "localhost";
my $ldapport = 389;
my $ldap;
my $bakdir = "/tmp/";
my $update_everything = "";
my $update_users = "";
my $update_dns = "";
my $update_mailboxtransport = "";
my $update_dhcp = "";
my $update_cyrus = "";

if( $#ARGV >= 0 ) {
    GetOptions("base=s" => \$baseDN,
	       "passwd=s" => \$CYRUSPW,
	       "users" => \$update_users,
	       "dns" => \$update_dns,
	       "mailtransport" => \$update_mailboxtransport,
	       "dhcp" => \$update_dhcp,
	       "cyrus" => \$update_cyrus,
	       "all" => \$update_everything
	       );
} else {
    do_help();
    exit;
}

if( $baseDN eq ""){
    print STDERR "The option --base is required\n";
    do_help();
    exit;
}
if( $CYRUSPW eq ""){
    print STDERR "The option --passwd is required\n";
    do_help();
    exit;
}

my $admin_dn = "uid=cyrus,".$baseDN;
my @dflt_timezone = parse_file("/etc/sysconfig/clock", "TIMEZONE=");
my $default_timezone = $dflt_timezone[0];
$default_timezone =~ s/\"//g;

$ldap = Net::LDAP->new($ldaphost, port => $ldapport, version => 3 );
my $res = $ldap->bind( $admin_dn, password => $CYRUSPW );
if( $res->code != 0 ){
    print $res->error()." ".$res->server_error()."\n";
    die("LDAP authentication failed!");
}
if($update_everything){   
    $update_users = 1;
    $update_dns = 1;
    $update_mailboxtransport = 1;
    $update_dhcp = 1;
    $update_cyrus = 1;
}

if($update_cyrus){
    print STDERR "Updating the cyrus object\n";
    update_cyrus();
    print STDERR "\n";
}
if($update_users){
    print STDERR "Updating the users objects\n";
    update_users();
    print STDERR "\n";
}
if($update_dns){
    print STDERR "Updating the DNS Configuration objects\n";
    update_dns();
    print STDERR "\n";
}
if($update_mailboxtransport){
    print STDERR "Creating MailTransport Entries\n";
    createMailTransport();
    print STDERR "\n";
}
if($update_dhcp){
    print STDERR "Creating DHCP setup\n";
    update_dhcp();
    print STDERR "\n";
}
exit 0;

sub do_help{
    print STDERR "\nldapupdated40.pl --base <ldapbase> --passwd <cyrus-password>\n\n";
    print STDERR "   additional parameters\n";
    print STDERR "      --cyrus          update the cyrus entry\n";
    print STDERR "      --ldap           update the user objects\n";
    print STDERR "      --dns            update the DNS configuration\n";
    print STDERR "      --dhcp           create the basic DHCP entries\n";
    print STDERR "      --mailransport   create the basic mailtransport entries\n";
    print STDERR "      --all            run all updates at once\n\n";
}


## do the real work
sub update_cyrus{
    my $ACI = "cn initials mail title ou l birthday description street postalcode ".
	      "st c susetimezone homephone mobile pager facsimiletelephonenumber ".
	      "telephonenumber labeleduri sn givenname jpegphoto logindestination";
    my $res = $ldap->search( base => "uid=cyrus,$baseDN",
			scope => "base",
			filter => "(objectclass=*)",
			attrs => ['defaultuseraci', "groupwaredbserver", "groupwaredbport"]
		    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading cyrus object\n";
	LDAPerror($res);
	return;
    }elsif($res->count != 1){
	print STDERR "Error reading cyrus object. Incorrect basedn?\n";
	LDAPerror($res);
	return;
    }
    my $entry = $res->entry(0);
    my $update=0;
    my $attr;
    $attr = $entry->get_value("defaultuseraci");
    if( (! $attr) || ($attr eq "") ){
	print STDERR "    adding \"defaultUserACI\"\n";
	$entry->add("defaultUserAci" => $ACI);
	$update=1;
    }else{  
	print STDERR "    \"defaultUserACI\"-Attribute already present.\n";
    }
    $attr = $entry->get_value("groupwaredbserver");
    if( (! $attr) || ($attr eq "") ){
	print STDERR "    adding \"groupwaredbserver\"\n";
	$entry->add("groupwaredbserver" => "localhost");
	$update=1;
    }else{  
	print STDERR "    \"groupwareDBserver\"-Attribute already present.\n";
    }
    $attr = $entry->get_value("groupwaredbport");
    if( (! $attr) || ($attr eq "") ){
	print STDERR "    adding \"groupwaredbport\"\n";
	$entry->add("groupwaredbport" => "5432");
	$update=1;
    }else{  
	print STDERR "    \"groupwareDBport\"-Attribute already present.\n";
    }
    if($update){
	$res = $entry->update( $ldap );
	if( $res->code != 0){
	    print STDERR "    Error updating User ".$entry->dn()."\n";
	    LDAPerror($res);
	}
    }
    return;
}

sub update_users{
    # construct the default aci first
    my $res = $ldap->search( base => "uid=cyrus,$baseDN",
			scope => "base",
			filter => "(objectclass=*)",
			attrs => ['defaultuseraci']
		    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading cyrus object\n";
	LDAPerror($res);
	return;
    }elsif($res->count != 1){
	print STDERR "Error reading cyrus object. Incorrect basedn?\n";
	LDAPerror($res);
	return;
    }
    my $default_useraci = $res->entry(0)->get_value("defaultuseraci");
    if( (! $default_useraci) || ($default_useraci eq "") ){
	print STDERR "The cyrus object doesn't contain a defaultUserACI-Attribute\n";
	print STDERR "Please run this script with --cyrus first\n";
	return;
    }
    my $aci = "1#entry#grant;r,w,s,c;";
    $default_useraci =~ s/ /,/g;
    $aci .= $default_useraci.";r,s,c;[all]#self#";
    
    # read all users
    $res = $ldap->search( base => $baseDN,
		   scope => "one",
		   filter => "(&(objectClass=posixAccount)(objectClass=SuSEemailObject))",
		   attrs => [ "uid" , "suseTimezone", "openldapaci"]
		);
    if( $res->code != 0 ){
	print STDERR "Error reading User objects\n";
	LDAPerror($res);
	return;
    }

    my @users = $res->entries();

    foreach my $act_user ( @users ){
	my $update = 0;
	my $uid = $act_user->get_value("uid");
	my $timezone = $act_user->get_value("susetimezone");

	if( (! $timezone) || ($timezone eq "") ){
	    $act_user->add( "susetimezone" => "$default_timezone" );
	    $update = 1;
	}
	
	my $useraci = $act_user->get_value("openldapaci");
	if( (! $useraci) || ($useraci eq "") ){
	    $act_user->add( "openldapaci" => "$aci" );
	    $update = 1;
	}

	my $up_res = undef;
	if( $update ){
	    print STDERR "    Updating user: ".$uid."\n";
	    $up_res = $act_user->update( $ldap );
	    if( $up_res->code != 0){
		print STDERR "    Error updating User ".$act_user->dn()."\n";
		LDAPerror($up_res);
	    }
	}else{
	    print STDERR "    User $uid does not need an update\n";
	}
    }
}

sub update_dns{
    ## backup DNS config to a LDIF
    my $res = $ldap->search( base => "o=DNS,".$baseDN,
			     scope => "one",
			     filter => "(objectclass=*)"
			    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading DNS configuration\n";
	LDAPerror($res);
	return;
    }elsif($res->count == 0){
	print STDERR "No DNS configuration in LDAP\n";
	return;
    } 
    my ($fh, $filename) = tempfile( "$bakdir/dnszone.XXXXXX", SUFFIX => ".ldif");
    my $ldif = Net::LDAP::LDIF->new($fh);
    $ldif->write_entry($res->entries());
    $ldif->done();

    ## get all zones and create corresponding ou's
    print STDERR "Reading DNS configuration\n";
    $res = $ldap->search( base => "relativeDomainName=@,o=DNS,".$baseDN,
			     scope => "base",
			     filter => "(objectclass=dnsZone)",
			    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading DNS configuration\n";
	LDAPerror($res);
	if($res->code == LDAP_NO_SUCH_OBJECT){
	    return;
	}
    }elsif($res->count == 0){
	print STDERR "No DNS configuration in LDAP\n";
	return;
    }
    my $entry = $res->entry(0);
    my @zones = $entry->get_value("zonename");
    my $soarec = $entry->get_value("sOARecord");
    my $nsrec = $entry->get_value("nSRecord");
    my $mxrec = $entry->get_value("mXRecorD");
    my $dnsttl = $entry->get_value("dNSTTL");
    print STDERR "Creating Zones\n";
    foreach my $zone ( @zones ){
	my $zoneOU = Net::LDAP::Entry->new;
	$zoneOU->dn("ou=".$zone.",o=DNS,$baseDN");
	$zoneOU->add('objectclass' => 'top',
		     'objectclass' => 'organizationalUnit',
		     'ou' => $zone,
		    );
	print STDERR "    adding ".$zoneOU->dn()."\n";
	$res = $ldap->add($zoneOU);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$zoneOU->dn()."\n";
	    LDAPerror($res);
	}

	my $zoneEntry = Net::LDAP::Entry->new;
	$zoneEntry->dn("relativeDomainName=@,ou=$zone,o=DNS,$baseDN");
	$zoneEntry->add('objectclass' => 'top',
			'objectclass' => 'dNSZone',
			'relativeDomainName' => '@',
			'zoneName' => $zone,
			'sOARecord' => $soarec,
			'nSRecord' => $nsrec,
			'mxRecord' => $mxrec,
			'dNSTTL' => $dnsttl
		       );
	if($zone !~ /\.IN-ADDR\.ARPA$/i ){
	    $zoneEntry->add('objectclass' => 'SuSEVirtualDomainObject',
			    'MTALocaldomain' => 'true');
	}
	print STDERR "    adding ".$zoneEntry->dn()."\n";
	$res = $ldap->add($zoneEntry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$zoneEntry->dn()."\n";
	    LDAPerror($res);
	}
    }
    print STDERR "\n";
    print STDERR "Reading DNS Hosts\n";
    $res = $ldap->search( base => "o=DNS,".$baseDN,
			     scope => "one",
			     filter => '(&(objectclass=dnsZone)(!(relativedomainname=@)))'
			    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading DNS configuration\n";
	LDAPerror($res);
    }elsif($res->count == 0){
	print STDERR "No DNS configuration in LDAP\n";
	return;
    }
    my @entries = $res->entries();
    foreach my $entry (@entries){
	my $zonename = $entry->get_value("zonename");
	my $rdn = $entry->get_value("relativedomainname");
	my $dnsttl = $entry->get_value("dnsttl");
	my $ptrrec = $entry->get_value("ptrrecord");
	my $arec = $entry->get_value("arecord");
	my $newentry = Net::LDAP::Entry->new;
	$newentry->dn("relativedomainname=$rdn,ou=$zonename,o=DNS,$baseDN");
	$newentry->add('objectclass' => 'top',
			'objectclass' => 'dnszone',
			'relativedomainname' => $rdn,
			'zonename' => $zonename,
			'dNSTTL' => $dnsttl,
		       );
	if($arec){
	    $newentry->add('aRecord' => $arec);
	}
	if($ptrrec){
	    $newentry->add('pTRRecord' => $ptrrec);
	}
	print STDERR "    adding ".$newentry->dn()."\n";
	$res = $ldap->add($newentry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error updating ".$newentry->dn()."\n";
	    LDAPerror($res);
	}else{
	    print STDERR "    deleting ".$entry->dn()."\n";
	    $res = $ldap->delete($entry->dn());
	    if( $res->code != LDAP_SUCCESS ){
		print STDERR "error deleting ".$newentry->dn()."\n";
		LDAPerror($res);
	    }
	}
    }
    print STDERR "removing unneeded entry: relativedomainname=@,"."o=dns,$baseDN\n";
    $res = $ldap->delete('relativedomainname=@,'."o=dns,$baseDN");
}

sub createMailTransport{
    # get the default domain
    my $res = $ldap->search( base => "uid=cyrus,$baseDN",
			scope => "base",
			filter => "(objectclass=*)",
			attrs => ['mailDomain']
		    );
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "Error reading cyrus object\n";
	LDAPerror($res);
	return;
    }elsif($res->count != 1){
	print STDERR "Error reading cyrus object. Incorrect basedn?\n";
	LDAPerror($res);
	return;
    }
    my $mailDomain = $res->entry(0)->get_value("mailDomain");
    #check if the ou=MailTransport-Object is already there
    $res = $ldap->search( base => "ou=MailTransports,$baseDN",
			  scope => "base",
			  filter => "(objectclass=*)");
    if($res->code == LDAP_NO_SUCH_OBJECT){
	my $mtEntry = Net::LDAP::Entry->new();
	$mtEntry->dn("ou=MailTransports,$baseDN");
	$mtEntry->add("objectclass" => "organizationalUnit",
		      "objectclass" => "top",
		      "ou" => "MailTransports"
		     );
	print STDERR "Adding ou=MailTransports,$baseDN\n";
	$res = $ldap->add($mtEntry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$mtEntry->dn()."\n";
	    LDAPerror($res);
	    return;
	}
    }elsif( $res->code != LDAP_SUCCESS ){
	print STDERR "Error while looking for Transport Tables\n";
	LDAPerror($res);
    }elsif($res->count == 1 ){
	print STDERR "    The ou=MailTransports entry is already present, nothing to do here.\n";
    }
    $res = $ldap->search( base => "cn=ferrariFax,ou=MailTransports,$baseDN",
			  scope => "base",
			  filter => "(objectclass=*)");
    if( $res->code == LDAP_NO_SUCH_OBJECT ){
	my $entry = Net::LDAP::Entry->new();
	$entry->dn("cn=ferrariFax,ou=MailTransports,$baseDN");
	$entry->add("objectclass" => "top",
		    "objectclass" => "SuSEMailTransportObject",
		    "cn" => "ferrariFax",
		    "smtpDomain" => "ums.$mailDomain",
		    "smtpDomain" => "fax.$mailDomain",
		    "smtpDomain" => "sms.$mailDomain",
		    "smtpDomainTransportNextHop" => "smtp:[localhost:2519]"
		     );
	print STDERR "Adding cn=ferrariFax,ou=MailTransports,$baseDN\n";
	$res = $ldap->add($entry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$entry->dn()."\n";
	    LDAPerror($res);
	    return;
	}
    }elsif( $res->code != LDAP_SUCCESS ){
	print STDERR "Error while looking for Transport Tables\n";
	LDAPerror($res);
    }elsif($res->count >= 1 ){
	print STDERR "    The cn=ferrariFax,ou=MailTransports entry is already present, nothing to do here.\n";
    }
}

sub update_dhcp{
    my $res = $ldap->search( base => "o=dhcp,$baseDN",
			  scope => "base",
			  filter => "(objectclass=*)");
    if($res->code == LDAP_NO_SUCH_OBJECT){
	my $entry = Net::LDAP::Entry->new();
	$entry->dn("o=DHCP,$baseDN");
	$entry->add("objectclass" => "organization",
		      "objectclass" => "top",
		      "o" => "DHCP"
		     );
	print STDERR "    Adding o=DHCP,$baseDN\n";
	$res = $ldap->add($entry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$entry->dn()."\n";
	    LDAPerror($res);
	    return;
	}
    }elsif( $res->code != LDAP_SUCCESS ){
	print STDERR "Error while looking for DHCP Entries\n";
	LDAPerror($res);
    }elsif($res->count == 1 ){
	print STDERR "    The o=DHCP entry is already present, nothing to do here.\n";
    }
    $res = $ldap->search( base => "cn=DHCP,o=DHCP,$baseDN",
			  scope => "base",
			  filter => "(objectclass=*)");
    if($res->code == LDAP_NO_SUCH_OBJECT){
	my $entry = Net::LDAP::Entry->new();
	$entry->dn("cn=DHCP,o=DHCP,$baseDN");
	$entry->add("objectclass" => "top",
		    "objectclass" => "dhcpServer",
		    "cn" => "DHCP",
		    "dhcpServiceDn" => "cn=Global,cn=DHCP,o=DHCP,$baseDN"
		   );
	print STDERR "    Adding cn=DHCP,o=DHCP,$baseDN\n";
	$res = $ldap->add($entry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$entry->dn()."\n";
	    LDAPerror($res);
	    return;
	}
    }elsif( $res->code != LDAP_SUCCESS ){
	print STDERR "Error while looking for DHCP Entries\n";
	LDAPerror($res);
    }elsif($res->count == 1 ){
	print STDERR "    The cn=DHCP,o=DHCP entry is already present, nothing to do here.\n";
    }
    $res = $ldap->search( base => "cn=Global,cn=DHCP,o=DHCP,$baseDN",
			  scope => "base",
			  filter => "(objectclass=*)");
    if($res->code == LDAP_NO_SUCH_OBJECT){
	my $entry = Net::LDAP::Entry->new();
	$entry->dn("cn=Global,cn=DHCP,o=DHCP,$baseDN");
	$entry->add("objectclass" => "top",
		    "objectclass" => "dhcpService",
		    "objectclass" => "dhcpOptions",
		    "cn" => "Global",
		    "dhcpPrimaryDn" => "cn=DHCP,o=DHCP,$baseDN"
		   );
	print STDERR "    Adding ".$entry->dn()."\n";
	$res = $ldap->add($entry);
	if( $res->code != LDAP_SUCCESS ){
	    print STDERR "error creating ".$entry->dn()."\n";
	    LDAPerror($res);
	    return;
	}
    }elsif( $res->code != LDAP_SUCCESS ){
	print STDERR "Error while looking for DHCP Entries\n";
	LDAPerror($res);
    }elsif($res->count == 1 ){
	print STDERR "    The cn=Global,cn=DHCP,o=DHCP entry is already present, nothing to do here.\n";
    }
}

sub LDAPerror($){
    my $res = shift;
    if( $res->code != LDAP_SUCCESS ){
	print STDERR "    Error message:".$res->error()."\n";
	if($res->server_error()){
	    print STDERR "    Additional info:".$res->server_error()."\n";
	}
    }
}
